import { Provide, Inject } from '@midwayjs/decorator';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { ReturnModelType } from '@typegoose/typegoose';
import { BaseService } from 'free-midway-component';
import { Role } from '../../entity/system/role.entity';
import { Menus } from '../../entity/system/menus.entity';
import { MenusService } from './menus.service';
import { flattenDeep, uniq, get, lowerFirst, upperFirst, last, compact } from 'lodash';
import { Context } from '@midwayjs/koa';
import { I18nService } from '../i18n.service';
import { Admin } from '../../entity/system/admin.entity';
import { Dept } from '../../entity/system/dept.entity';
type modelType = ReturnModelType<typeof Role>;
@Provide()
export class RoleService extends BaseService<modelType> {
  @InjectEntityModel(Role)
  model: modelType;
  @InjectEntityModel(Menus)
  menusModel: ReturnModelType<typeof Menus>;

  @InjectEntityModel(Admin)
  adminModel: ReturnModelType<typeof Admin>;

  @InjectEntityModel(Dept)
  deptModel: ReturnModelType<typeof Dept>;

  @Inject()
  menusService: MenusService;

  @Inject()
  ctx: Context;
  @Inject()
  i18n: I18nService;
  //是否是超级管理员
  async isSuperAdmin() {
    const user = this.ctx.user;
    const admin = await this.adminModel.findById(user._id);
    if (!admin) return false;
    if (admin.is_super === '0') return true;
  }
  /**
   * 获取用户接口权限编码列表
   * @returns {Array<string>} apiCodes 用户接口权限编码列表
   */
  async getUserApiCodes() {
    const user = this.ctx.user;
    const getControllerCode = list => {
      const result = [];
      for (const i of list) {
        const { config = [], children = [] } = i;
        const l = config.map(i => get(i, 'controller_code'));
        result.push(...l);
        if (children && children.length > 0) {
          const cl = getControllerCode(children);
          result.push(...cl);
        }
      }
      return result;
    };
    if (await this.isSuperAdmin()) {
      const menus = await this.menusService.queryMenu({ is_use: '0' });
      // 需要整理出所有 config 中 methodCode = ${controllerCode}.${method} ,query 没有二级code
      let apiCodes = getControllerCode(menus);
      apiCodes = compact(apiCodes);
      return apiCodes;
    }
    // 其他用户需要把 部门 和 角色的权限叠加在一起
    // 部门权限
    const deptCodes = await this.getDeptCodes(get(user, 'dept'));
    // 角色权限
    const roleCodes = await this.getRoleCodes(get(user, 'role'));
    let allCodes = [...deptCodes, ...roleCodes];
    allCodes = uniq(allCodes);
    const menuList = await this.menusModel.find({ route_name: allCodes }).lean();
    const apiCodes = getControllerCode(menuList);
    return apiCodes;
  }

  async getUserMenus(needCode = false) {
    const user = this.ctx.user;
    // 平台的超级管理员有所有权限
    if (await this.isSuperAdmin()) {
      const menus = await this.menusService.queryMenu({ is_use: '0' });
      return { menus };
    }
    // 其他用户需要把 部门 和 角色的权限叠加在一起
    // 部门权限
    const deptCodes = await this.getDeptCodes(get(user, 'dept'));

    // 角色权限
    const roleCodes = await this.getRoleCodes(get(user, 'role'));
    let allCodes = [...deptCodes, ...roleCodes];
    allCodes = uniq(allCodes);
    // 只获取编码,直接返回
    if (needCode) return allCodes;
    // 根据编码获取菜单数据
    const menuCodes = this.setMenuCodes(allCodes);
    const menuList = await this.menusModel.find({ route_name: menuCodes }).lean();
    let treeMenu = menuList.filter(f => !f.parent_id);
    treeMenu = this.menusService.treeMenu(menuList, treeMenu);
    return { menus: treeMenu, role_code: allCodes };
  }

  /**
   * 整理出角色菜单的编码,主要是与route_name编码对应
   * @param allCodes 本用户的角色+部门的完整权限编码
   * @returns 菜单需要的编码
   */
  setMenuCodes(allCodes: Array<string>) {
    const result = [];
    for (const code of allCodes) {
      const arr = code.split('.');
      result.push(last(arr));
    }
    return uniq(result);
  }

  /**
   * 根据部门id获取权限列表
   * @param deptId 部门id
   * @returns 权限列表
   */
  async getDeptCodes(deptId: string) {
    const dept = await this.deptModel.findById(deptId).lean();
    if (!dept) return [];
    // 处理部门有的权限
    const resource = get(dept, 'resource', []);
    // const result = resource.map(i => {
    //   const arr = i.split('.');
    //   return last(arr);
    // });
    return resource;
  }
  /**
   * 根据角色数组获取角色权限
   * @param roleCode 角色数据
   * @returns 权限列表
   */
  async getRoleCodes(roleCode: Array<string>) {
    const roleCodes = [];
    for (const rc of roleCode) {
      roleCodes.push(lowerFirst(rc));
      roleCodes.push(upperFirst(rc));
    }
    const role = await this.model.findOne({ code: roleCode, is_use: '0' }).lean();
    if (!role) return [];
    const roleMenu = get(role, 'menu', []);
    // const result = roleMenu.map(i => {
    //   const arr = i.split('.');
    //   return last(arr);
    // });
    return roleMenu;
  }

  /**
   * 根据角色id数组,整理菜单
   * @param roleIdList 角色id数组
   */
  async getMenuByRoles(roleIdList) {
    let menus = [];
    // 第一步,整理所有菜单至一维数组
    for (const r of roleIdList) {
      menus.push(...flattenDeep(r));
    }
    menus = uniq(menus);
    const allMenu = await this.menusModel.find({ _id: menus, is_use: '0' }, { meta: 0, __v: 0 }).sort({ order_num: 1 }).lean();
    let treeMenu = allMenu.filter(f => !f.parent_id);
    treeMenu = this.menusService.treeMenu(allMenu, treeMenu);
    return treeMenu;
  }
}
